/***************************************************************************
 *   Copyright (C) 2004 by Eugene Bort                                     *
 *   esvn@umputun.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
  ***************************************************************************/
  
#include "dirs_list.h"

bool C_DirsList::load(const QString& path, QListView* list_dirs, QListViewItem* root, bool open, bool disable_symlinks, bool append) 
{

	QListViewItem* root_item = 0;

	if (!root)  {//wihtout parent it means top-level directory
		if (!append) 
			list_dirs->clear();

		if (root == NULL) {
			root_item = new QListViewItem(list_dirs, 0);
			QString short_name = path;
			int pos = short_name.findRev('/');
			if (pos != -1) 
				short_name = path.mid(pos+1, 64);

			//root_item->setPixmap(0, QPixmap::fromMimeSource( "folder_home_2.png" ) );
			root_item->setPixmap(0, QPixmap::fromMimeSource("folder_green_open.png")); //colorscm.png") );

			root_item->setText(1, path);
			root_item->setText(0, short_name);
			load(path, list_dirs, root_item);
			//root_item->setOpen(TRUE);
			return true;
		}

	}


	//performs open/close and change folder icon
	if (root) { //if parent defined, 
		if (root->childCount() >0 && root->firstChild()->text(1) != "empty item") {
			if ( isSvnControlled(root->text(1)) ){
				if (root->isOpen())
					root->setPixmap(0, QPixmap::fromMimeSource( "folder_green_open.png" ) );
				else
					root->setPixmap(0, QPixmap::fromMimeSource( "folder_green.png" ) );
			}
			else {
				if (root->isOpen())
					root->setPixmap(0, QPixmap::fromMimeSource( "folder_grey_open.png" ) );
				else
					root->setPixmap(0, QPixmap::fromMimeSource( "folder_grey.png" ) );
			}
			return true;
		}

	}


	if (root->childCount() == 1 && root->firstChild()->text(1) == "empty item")
		root->takeItem(root->firstChild());

	QDir workspace_dir = (root == NULL) ? QDir(path) : QDir(root->text(1));
	workspace_dir.setFilter(QDir::Dirs | QDir::Hidden | (disable_symlinks ? QDir::NoSymLinks : 0) );

	if (!workspace_dir.exists() ) {
		qWarning( ("Cannot find " + path).local8Bit());
		return false;
	}

	for (size_t i=0; i<workspace_dir.count(); i++) {

		QString curr = workspace_dir[i];
		if ( curr == "." || curr == ".." || curr == ".svn")
			continue;


		QListViewItem* item;
		if (root == NULL) {
			item = new QListViewItem(list_dirs, root_item);
			item->setText(1, path + "/" + curr);
		}
		else {
			item = new QListViewItem(root, 0);
			root->setOpen(open);

			if ( isSvnControlled(root->text(1) ) )
				root->setPixmap(0, QPixmap::fromMimeSource( "folder_green_open.png" ) );
			else
				root->setPixmap(0, QPixmap::fromMimeSource( "folder_grey_open.png" ) );

			item->setText(1, root->text(1) + "/" + curr);

			if (hasSubDirs(root->text(1) + "/" + curr, disable_symlinks)) {
				QListViewItem *sub_item = new QListViewItem(item, 0);
				sub_item->setText(1, "empty item");
			}
		}

		item->setText(0, curr);

	    if (isSvnControlled(root->text(1) + "/" + curr))
			item->setPixmap(0, QPixmap::fromMimeSource( "folder_green.png" ) );
		else 
			item->setPixmap(0, QPixmap::fromMimeSource( "folder_grey.png" ) );

	}


	return true;
}


bool C_FilesList::load(const QString& path, QListView* list_files, bool hide_unknown, bool show_dirs, map<QString, QString>* rev_list) 
{
	list_files->clear();

	QDir files =  QDir(path);
	if (show_dirs) 
		files.setFilter(QDir::Files | QDir::Hidden | QDir::Dirs );
	else
		files.setFilter(QDir::Files | QDir::Hidden);

	if (!files.exists() ) {
		qWarning( ("Cannot find the " + path).local8Bit());
		return false;
	}

	svn_entries.clear();
	svn_entries.setDir(path, show_dirs);

	C_SvnEntries curr_svn_entries = svn_entries;

	//qApp->processEvents();

	for (size_t i=0; i<files.count(); i++) {

		//qApp->processEvents(QEventLoop::ExcludeUserInput);

		QString curr = files[i];
		if (curr == "." || curr == ".." || curr ==".svn") 
			continue;

		C_SvnFile svn_file_info = curr_svn_entries.get(curr);

		if (!svn_file_info.is_svn_controled_ && hide_unknown) 
			continue;

		curr_svn_entries.erase(curr);

		C_NumSortedFileListItem* item = new C_NumSortedFileListItem(list_files, 0);
		item->setText(0, curr);


		if (svn_file_info.is_svn_controled_) {
			if (svn_file_info.modif_dtm_ == svn_file_info.text_dtm_) {
				QString full_name = svn_file_info.name_;
				if (!path.isEmpty()) 
					full_name = path + "/" + svn_file_info.name_;

				if (rev_list && rev_list->find(full_name) != rev_list->end() && rev_list->find(full_name)->second != svn_file_info.commited_revision_) {

					if (svn_file_info.kind_ == QString("dir") ) {				
						item->setPixmap(0, QPixmap::fromMimeSource( "folder_exec_3.png")) ;
						item->setDirType();
					} 
					else
						item->setPixmap(0, QPixmap::fromMimeSource( "FileNeedsPatch16x16.xpm" ) );

					item->setText(1, "out-of-date");
				}
				else {
					if (svn_file_info.kind_ == QString("dir") ) {				
						item->setPixmap(0, QPixmap::fromMimeSource( "folder_green.png")) ;
						item->setDirType();
					} 
					else 
						item->setPixmap(0, QPixmap::fromMimeSource( "FileWriteable16x16.xpm" ) );
					item->setText(1, "up-to-date");
				}
			}
			else {
				if (svn_file_info.kind_ == QString("dir") ) {				
					item->setPixmap(0, QPixmap::fromMimeSource( "folder_red.png")) ;
					item->setDirType();
				} 
				else 
					item->setPixmap(0, QPixmap::fromMimeSource( "FileModified16x16.xpm" ) );
				item->setText(1, "modified");
			}

			if (svn_file_info.schedule_ == "add") {
				if (svn_file_info.kind_ == QString("dir") ) {				
					item->setPixmap(0, QPixmap::fromMimeSource( "folder_orange.png")) ;
					item->setDirType();
				} 
				else 
					item->setPixmap(0, QPixmap::fromMimeSource( "FileAdded16x16.xpm" ) );

				item->setText(1, "add");
			}


			if (svn_file_info.schedule_ == "delete") {
				if (svn_file_info.kind_ == QString("dir") ) {				
					item->setPixmap(0, QPixmap::fromMimeSource( "folder_red_3.png")) ;
					item->setDirType();
				}
				else
					item->setPixmap(0, QPixmap::fromMimeSource( "FileRemoved16x16.xpm" ) );

				item->setText(1, "removed");
			}

			if (svn_file_info.conflict_wrk_ != "") {
				item->setPixmap(0, QPixmap::fromMimeSource( "FileConflict16x16.xpm" ) );
				item->setText(1, "conflict");
			}

			item->setText(2, svn_file_info.revision_);
			item->setText(3, svn_file_info.commited_revision_);
			item->setText(4, C_SvnEntries::UTC2Local(svn_file_info.commited_dtm_).toString("yy/MM/dd hh:mm:ss"));

			if (svn_file_info.modif_dtm_.isValid() )
				item->setText(5, C_SvnEntries::UTC2Local(svn_file_info.modif_dtm_).toString("yy/MM/dd hh:mm:ss"));

			item->setText(6, svn_file_info.last_author_);

			//cout <<"tb:" <<  svn_file_info.tag_ << " " << svn_file_info.branch_ << endl;
			if (svn_file_info.tag_ != "") {
				item->setText(7, svn_file_info.tag_);
				item->setPixmap(7, QPixmap::fromMimeSource("t-16x16.xpm" ) );
			}

			if (svn_file_info.branch_ != "") {
				item->setText(7, svn_file_info.branch_);
				item->setPixmap(7, QPixmap::fromMimeSource("b-16x16.xpm" ) );
			}

			if (svn_file_info.branch_ == "" && svn_file_info.tag_ == "")
				item->setText(7, "trunk");

		}
		else {
			if (svn_file_info.kind_ == QString("dir") ) {
				item->setPixmap(0, QPixmap::fromMimeSource( "folder_grey.png")) ;
				item->setDirType();
			}
			else 
				item->setPixmap(0, QPixmap::fromMimeSource( "FileUnknown16x16.xpm" ) );

			item->setText(1, "unknown");
		}


		item->setText(16, path + "/" + curr);

	}

	//cout << "rest:" << curr_svn_entries.getData().size() << endl;;

	map<QString, C_SvnFile>::iterator iter_entries;
	for (iter_entries = curr_svn_entries.getData().begin(); iter_entries != curr_svn_entries.getData().end(); iter_entries++) {
		C_SvnFile svn_file_info = iter_entries->second;

		if (!show_dirs && svn_file_info.kind_ != "file") 
			continue;

		if (!svn_file_info.is_svn_controled_ || svn_file_info.name_ == "" || svn_file_info.name_ == "." || svn_file_info.name_ == ".."  || svn_file_info.name_ == ".svn"  || /*svn_file_info.kind_ != "file" ||*/ svn_file_info.deleted_ == "true") 
			continue;



		C_NumSortedFileListItem* item = new C_NumSortedFileListItem(list_files, 0);
		item->setText(0, svn_file_info.name_);

		if (svn_file_info.schedule_ == "delete") {
			if (svn_file_info.kind_ == QString("dir") ) {				
				item->setPixmap(0, QPixmap::fromMimeSource( "folder_orange.png")) ;
				item->setDirType();
			}
			else
				item->setPixmap(0, QPixmap::fromMimeSource( "FileRemoved16x16.xpm" ) );

			item->setText(1, "removed");
		}
		else {
			if (svn_file_info.is_svn_controled_) {
				if (svn_file_info.kind_ == QString("dir") ) {				
					item->setPixmap(0, QPixmap::fromMimeSource( "folder_violet.png")) ;
					item->setDirType();
				}
				else
					item->setPixmap(0, QPixmap::fromMimeSource( "FileNeedsCheckout16x16.xpm" ) );
				item->setText(1, "missing");
			}
		}

		item->setText(2, svn_file_info.revision_);
		item->setText(3, svn_file_info.commited_revision_);

		item->setText(4, C_SvnEntries::UTC2Local(svn_file_info.commited_dtm_).toString("yy/MM/dd hh:mm:ss"));

		if (svn_file_info.modif_dtm_.isValid())
			item->setText(5, C_SvnEntries::UTC2Local(svn_file_info.modif_dtm_).toString("yy/MM/dd hh:mm:ss"));

		item->setText(6, svn_file_info.last_author_);

	}
	return true;
}

bool C_FilesList::cmp(C_SvnEntries sec)
{
	bool cmp_res = true;

	//cout << "cmp: " << svn_entries.getData().size() << " " << sec.getData().size() << endl;

	if (svn_entries.getData().size() == sec.getData().size()) {
		map<QString, C_SvnFile>::iterator iter_svn;
		for (iter_svn = svn_entries.getData().begin(); iter_svn != svn_entries.getData().end();  iter_svn++) {

			if (sec.getData().find(iter_svn->first) == svn_entries.getData().end()) {
				cmp_res = false;
				//cout << "diff names" << endl;
				break;
			}

			C_SvnFile svn_1 = sec.getData()[iter_svn->first];
			C_SvnFile svn_2 = svn_entries.getData()[iter_svn->first];

			if (svn_1.name_ != svn_2.name_ || svn_1.modif_dtm_ != svn_2.modif_dtm_) {
				cmp_res = false;
				//cout << "diff data" << endl;
				break;
			}
		}
	}
	else {
		cmp_res = false;
		//cout << "diff size " << svn_entries.getData().size() << " " <<sec.getData().size() << endl;
	}

	return cmp_res;
}
