/***************************************************************************
 *   Copyright (C) 2004 by Eugene Bort                                     *
 *   ulang@umputun.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 ***************************************************************************/

#ifndef DIRS_LIST_H
#define DIRS_LIST_H

#include <qobject.h>
#include <qlistview.h>
#include <qdir.h>
#include <qtimer.h> 
#include <qpixmap.h> 
#include <qstring.h>
#include <qapplication.h> 
#include <qfileinfo.h>
#include <qeventloop.h>


#include <string>
#include <iostream>

#include "entries_parser.h"

using namespace std;

class C_DirsList : public QWidget {

	Q_OBJECT

public:
	C_DirsList() : hideUnknown_(false) {
		
	}

	bool load(const QString& path, QListView* list_dirs, QListViewItem* root, bool open=true, bool disable_symlinks=true, bool append=false);

	static QString removeLastLevel(const QString& full_file_name) {
		int pos = full_file_name.findRev('\\');
		if (pos == -1)
			pos = full_file_name.findRev('/');

		if (pos == -1)
			return full_file_name;
		else
			return full_file_name.left(pos);
	}

	void setHide(bool hide) {
		hideUnknown_ = hide;
	}

public slots:



private:
	QString getShortName(const QString& full_file_name) {
		int pos = full_file_name.findRev('\\');
		if (pos == -1)
			pos = full_file_name.findRev('/');
		
		if (pos == -1)
			return full_file_name;
		else
			return full_file_name.mid(pos+1, full_file_name.length()-1);
	}

	bool isSvnControlled(const QString& dir) {
		QFileInfo fi(dir + "/.svn/entries");
		return (fi.exists());
	}

	bool hasSubDirs(const QString &dir, bool disable_symlinks) {
		QDir curr_dir =  QDir(dir);
		curr_dir.setFilter(QDir::Dirs | QDir::Hidden | (disable_symlinks ? QDir::NoSymLinks : 0) );
		curr_dir.setPath(dir);

		if (!curr_dir.count()) 
			return false;

		for (size_t i=0; i<curr_dir.count(); i++) {
			if (curr_dir[i] != "." && curr_dir[i] != ".." && curr_dir[i] != ".svn")
				return true;
		}
		return false;

	}
	bool hideUnknown_;
};


class C_FilesList {
public:
	C_FilesList() {}
	bool load(const QString& path, QListView* list_files, bool hide_unknown, bool show_dirs, map<QString, QString>* rev_list);
	bool cmp(C_SvnEntries sec);
	C_SvnEntries svn_entries;
};


class C_NumSortedFileListItem : public QListViewItem {
public:
	C_NumSortedFileListItem( QListView * parent, QListViewItem * after ) :
		QListViewItem(parent, after) , is_dir_(false) {}

	void setDirType() {is_dir_ = true;}
	bool isDirType() {return is_dir_;}

	int compare(QListViewItem *i, int col, bool ascending ) const {
		if (col == 2 || col == 3) {
			if (text(col).toULong()  == i->text(col).toULong() )	
				return 0;
			return (text(col).toULong()  > i->text(col).toULong()) ? 1 : -1; 
		}

		if (col == 0) {
			if (is_dir_ && !((C_NumSortedFileListItem*)i)->isDirType()) 
				return -1;

			if (!is_dir_ && ((C_NumSortedFileListItem*)i)->isDirType()) 
				return 1;
		}

		return QListViewItem::compare(i, col, ascending);
	}


private:
	bool is_dir_;
};

#endif


