/***************************************************************************
 *   Copyright (C) 2004 by Eugene Bort                                     *
 *   esvn@umputun.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
  ***************************************************************************/

#include "entries_parser.h"

#include <qdom.h>


bool C_SvnFile::parse()
{
	if (xml_entry_ == "")
		return false;

	//fill file information
	QFileInfo fi( file_name_ );
	is_exist_ = fi.exists();
	if( is_exist_ ) {
		modif_dtm_ = fi.lastModified();
		time_t t = modif_dtm_.toTime_t();;
		modif_dtm_.setTime_t(t, Qt::UTC);
		size_ = fi.size();
		return true;
	}
    return false;
}


void C_SvnEntries::refresh(bool proc_subdirs)
{
	fillEntries(data_,  "", proc_subdirs);
	return;
}



void C_SvnEntries::fillEntries(map<QString, C_SvnFile> &data, const QString& dir, bool proc_subdirs)
{
	QString entr_file_name = ".svn/entries";
	if (!dir.isEmpty()) 
		entr_file_name = dir + "/" + entr_file_name;

	QFileInfo fi(entr_file_name);
	if (!fi.exists()) {
		is_svn_controled_ = false;
		return;
	}

	QDomDocument dom;
	QFile entries_file(entr_file_name);
	QString error_messsage;
	int error_line, error_column;
	if( !dom.setContent( &entries_file, &error_messsage, &error_line, &error_column ) ) {
		qWarning( "%s .svn/entries:%d:%d:%s", (const char*)dir_.local8Bit(), 
				  error_line, error_column, (const char*)error_messsage.local8Bit() );
		is_svn_controled_ = false;
		return;
	}

	C_SvnFile svn_file;
    QString base_revision, base_tag, base_branch;

	data.clear();

	QDomElement el;
	QDomNode node = dom.documentElement().firstChild();
	while ( !node.isNull() ) {
		if( node.isElement() ) {
			el = node.toElement();
			//qDebug( "reading element: %s: %s", el.tagName().latin1(), (const char*)el.attribute( "name" ).local8Bit() );
			if( el.tagName() == "entry" ) {
				svn_file.name_ = el.attribute( "name" );
				svn_file.kind_ = el.attribute( "kind");

				svn_file.commited_revision_ = el.attribute( "committed-rev");

				svn_file.url_ = el.attribute( "url");

				//process tag/branch info
				if (!svn_file.url_.isEmpty())
					getTagBranch(svn_file.url_, base_tag, base_branch);


				svn_file.last_author_ = el.attribute( "last-author");
				svn_file.uuid_ = el.attribute( "uuid");
				svn_file.schedule_ = el.attribute( "schedule");
				svn_file.deleted_ = el.attribute( "deleted");


				svn_file.conflict_wrk_ = el.attribute("conflict-wrk");
				svn_file.conflict_old_ = el.attribute("conflict-old");
				svn_file.conflict_new_ = el.attribute("conflict-new");

				svn_file.revision_ = el.attribute( "revision" );
				if (svn_file.name_.isEmpty())
						base_revision = svn_file.revision_;

				svn_file.commited_dtm_ = svn2dtm(el.attribute( "committed-date"));
				svn_file.text_dtm_ = svn2dtm(el.attribute("text-time"));

				if (proc_subdirs && svn_file.kind_ == "dir" && !svn_file.name_.isEmpty()) {
					map<QString, C_SvnFile> sub_data;
					fillEntries(sub_data, dir_+"/"+svn_file.name_, false);
					if (sub_data.find("") != sub_data.end()) {
						QString tmp = svn_file.name_;
						svn_file = sub_data[""];
						svn_file.name_ = tmp;

						//test for modif. files in directory
						map<QString, C_SvnFile>::iterator iter_sub;
						bool was_modif = false;

						for(iter_sub = sub_data.begin(); iter_sub != sub_data.end(); ++iter_sub) {
							C_SvnFile cur_f = iter_sub->second;
							if (cur_f.kind_ == "file"  && cur_f.is_svn_controled_ && !cur_f.name_.isEmpty() && cur_f.modif_dtm_ != cur_f.text_dtm_) {
								was_modif = true;
								break;
							}
						}
						
						if (!was_modif) {
							//cout << "up-2-date:" << svn_file.name_ << endl;
							QFileInfo fi( svn_file.name_ );
							if( fi.exists() ) {
								svn_file.text_dtm_ = fi.lastModified();
								time_t t = svn_file.text_dtm_.toTime_t();
								svn_file.text_dtm_.setTime_t(t, Qt::UTC);
							}
						}

					}
				}

	
				QString f_name = svn_file.name_;
				if (!dir.isEmpty())
					f_name = dir +"/" + f_name;
					
				QFileInfo fi(f_name );
				if( fi.exists() ) {
					svn_file.modif_dtm_ = fi.lastModified();
					time_t t = svn_file.modif_dtm_.toTime_t();
					svn_file.modif_dtm_.setTime_t(t, Qt::UTC);
					svn_file.size_ = fi.size();
				}
				else {
					//cout << "not here: " <<  f_name << endl;
				}

				svn_file.is_svn_controled_ = true;
				if (svn_file.revision_.isEmpty())
						svn_file.revision_ = base_revision;

				svn_file.branch_ = base_branch;
				svn_file.tag_ = base_tag;

				data[svn_file.name_] = svn_file;
			}
		}
		node = node.nextSibling();
	}


	//process rest of files (nonversioned)

	QDir files =  QDir(".");

	if (proc_subdirs) 
		files.setFilter(QDir::Files | QDir::Hidden | QDir::Dirs);
	else
		files.setFilter(QDir::Files | QDir::Hidden);

	for (size_t i=0; i<files.count(); i++) {
		QString curr = files[i];
		if (data_.find(curr) == data_.end()) {

			if (curr == "." || curr == "..") 
				continue;

			C_SvnFile svn_file;
			svn_file.is_svn_controled_ = false;
			svn_file.file_name_ = curr;
			svn_file.name_ = curr;
			QFileInfo fi(svn_file.name_);
			if( fi.exists() ) {
				svn_file.modif_dtm_ = fi.lastModified();
				time_t t = svn_file.modif_dtm_.toTime_t();;
				svn_file.modif_dtm_.setTime_t(t, Qt::UTC);
				svn_file.size_ = fi.size();
				if (fi.isDir() )
					svn_file.kind_ = "dir";
				else
					svn_file.kind_= "file";
			}
			data[svn_file.name_] = svn_file;
		}
	}

}




//2004-07-23T18:46:48.124692Z
QDateTime C_SvnEntries::svn2dtm(const QString& value)
{
    QDateTime ret;
	if (value.length() < 20) {
		//qWarning( "wrong time value: %s", value.local8Bit() );
		return ret;
	}

	
#if ( _WINDOWS || ESVN_WIN )

	/* The times on windows are not as accurate as those on unix os's, therefore
	 * the milliseconds needs to be removed. */
	QString value_mod(value);
	int pos = value_mod.findRev(".");
	value_mod.remove(pos, value_mod.length() - pos);
	ret = QDateTime::fromString(value_mod,Qt::ISODate);

#else
	ret = QDateTime::fromString(value,Qt::ISODate);
#endif
	return ret;
}

void C_SvnEntries::getTagBranch(const QString& url, QString& tag, QString& branch)
{

	tag = "";
	branch = "";

	int pos = url.find("/tags/");
	if (pos != -1) {
		int fn_pos = url.find("/", pos+6);
		if (fn_pos == -1) 
			fn_pos = url.length();
		tag = url.mid(pos+6, fn_pos - (pos+6));
		//cout <<"tag:"<<tag << endl;;
	}
	else {
		pos = url.find("/branches/");
		if (pos != -1) {
			int fn_pos = url.find("/", pos+10);
			if (fn_pos == -1) 
				fn_pos = url.length();
			branch = url.mid(pos+10, fn_pos - (pos+10));
		}
	}
}

