/***************************************************************************
 *   Copyright (C) 2004 by Eugene Bort                                     *
 *   esvn@umputun.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 ***************************************************************************/

#ifndef ENTRIES_PARSER_H
#define ENTRIES_PARSER_H

#include <sys/types.h>
#include <time.h>
#include <qstring.h>
#include <qdir.h>

#include <map>
#include <iostream>

using namespace std;

struct C_SvnFile {

	C_SvnFile() : xml_entry_(""), is_svn_controled_(false) {}

	C_SvnFile(const QString& xml_entry, const QString& file_name, const QString &path) 
		: xml_entry_(xml_entry), file_name_(file_name), path_(path){}

	QString xml_entry_;
	QString file_name_, path_;
	QString name_, url_;
	QString revision_, commited_revision_;
	QString kind_;
	QString uuid_;
	QString last_author_;
	QString schedule_;
	QString deleted_;

	QString tag_, branch_;
	QString conflict_wrk_, conflict_old_, conflict_new_;



	QDateTime commited_dtm_, modif_dtm_, text_dtm_;

	unsigned long size_;
	bool is_svn_controled_;
	bool is_exist_;

	void clean() {
		xml_entry_ = "";
		file_name_ = path_ = name_ = kind_ = schedule_ = deleted_ = "";
		revision_ = commited_revision_ = "";
		last_author_ = uuid_ = "";

		conflict_wrk_= conflict_old_ = conflict_new_ = "";
		last_author_ = uuid_ = tag_ = branch_ = "";

		commited_dtm_ = modif_dtm_ = QDateTime();
		size_ = 0;
	}

private:
	bool parse();

};


class C_SvnEntries {
public:
	C_SvnEntries() {}

	C_SvnFile get(QString file_name) {
		if (data_.find(file_name) == data_.end()) {
			C_SvnFile result;
			result.is_svn_controled_ = false;
			return result;
		}
		return data_[file_name];
	}

	void erase(QString file_name) {
		if (data_.find(file_name) != data_.end()) 
			data_.erase(data_.find(file_name));
	}

	bool setDir(const QString &dir, bool proc_subdirs) {
		dir_ = dir;
		if (QFileInfo(dir).exists()) {
			QDir::setCurrent( dir_ );
			//cout << "chdir: " << dir_ << endl;
			refresh(proc_subdirs);
			return true;
		}
		else
			return false;
	}

	void refresh(bool proc_subdirs);

	void fillEntries(map<QString, C_SvnFile> &data, const QString& dir= "", bool proc_subdirs=false);
	void getTagBranch(const QString& url, QString& tag, QString& branch);

	void clear() {
		data_.clear();
	}

	map<QString, C_SvnFile>& getData() {
		return data_;
	}

	static QDateTime UTC2Local(const QDateTime& utc) {

		QDateTime result(utc);

#if (__FreeBSD__ || __NetBSD__ || __OpenBSD__ || ESVN_BSD)
			time_t a; a = time(NULL);
			struct tm bsd_hack = *localtime(&a);
			result = result.addSecs(-(bsd_hack.tm_gmtoff - bsd_hack.tm_isdst*3600) );

#elif (_WINDOWS || ESVN_WIN)
			result = result.addSecs(-(_timezone - _daylight*3600));
#else
			result = result.addSecs(-(timezone - daylight*3600));
#endif

		return result;
	}


private:
	QDateTime svn2dtm(const QString& value);

	map<QString, C_SvnFile> data_;
	QString dir_;
	bool is_svn_controled_;
;
};

#endif

